#!/usr/bin/env python
# coding: utf-8
"""
python list_building_ids_and_addresses.py -h
usage: list_building_ids_and_addresses.py [-h] [file_or_folder]

Parses a CityGML file, lists Building IDs and Addresses if available,
and writes a CSV file along the CityGML.

positional arguments:
  file_or_folder  Path to citygml file, or path to folder containing multiple citygml files.
                  Defaults to "input" folder.

optional arguments:
  -h, --help      show this help message and exit
"""

from pathlib import Path
# pip install lxml
from lxml import etree

ID_TAG = '{http://www.opengis.net/gml}id'


def parse_citygml(citygml_filename):
    print(f'Extracting addresses from {citygml_filename}...')
    citygml_path = Path(citygml_filename)
    folder = citygml_path.parent
    basename = citygml_path.stem
    csv_path = folder / (basename + '_addresses.csv')
    root = etree.parse(str(citygml_filename)).getroot()
    addresses_count = 0
    with open(csv_path, 'w+', encoding='utf-8-sig') as csv_file:
        csv_file.write("# GML id;Address;Parent id\n")
        for building in root.findall('.//{*}Building'):
            building_id = building.get(ID_TAG)
            address_node = building.find('.//{*}Address')
            part_ids = [part.get(ID_TAG) for part in building.findall('.//{*}BuildingPart')]
            if address_node is None:
                address = ""
            else:
                every_address_info = [text.strip() for text in address_node.itertext()]
                address_info = [text for text in every_address_info if text]
                address = (','.join(address_info))
                addresses_count += 1
            csv_file.write(';'.join([building_id, address]))
            csv_file.write("\n")
            for part_id in part_ids:
                csv_file.write(';'.join([part_id, address, building_id]))
                csv_file.write("\n")

    print(f'  {csv_path} has been written (with {addresses_count} addresses).')


def parse_every_citygml(citygml_folder):
    print(f'Listing every citygml file in {citygml_folder} ...')
    citygmls = Path(citygml_folder).glob('*.gml')
    for citygml in citygmls:
        parse_citygml(citygml)
    print('Done')


if __name__ == '__main__':
    import argparse

    parser = argparse.ArgumentParser(description='Parses a CityGML file, lists Building IDs'
                                     ' and Addresses if available,'
                                     ' and writes a CSV file along the CityGML.')
    parser.add_argument("file_or_folder", help='Path to citygml file, or path to'
                        ' folder containing multiple citygml files.'
                        '\nDefaults to "input" folder.',
                        nargs='?', default='input')
    args = parser.parse_args()
    path = Path(args.file_or_folder)
    if path.exists():
        if path.is_dir():
            parse_every_citygml(path)
        elif path.suffix.lower() in ('.gml', '.xml'):
            parse_citygml(path)
        else:
            print(f'ERROR : Path "{path.resolve()}" should be a folder or a .gml/.xml file.')
    else:
        print(f'ERROR : Path "{path.resolve()}" should exist.')
